import time
import pretty_errors
import pandas as pd
import baostock as bs
from tqdm import tqdm
from .GetDate import GetDate
import datetime
import sys
import os
sys.path.append(os.path.dirname(__file__) + os.sep + '../')
from ..log.SlyLog import slog
from ..log.SlyLog import sprint

class StockData(object):


    def __init__(self, names=['贵州茅台', '隆基股份'],
                start_date='2020-12-01', end_date='2020-12-31',
                frequency="w",
                cache=False):
        self.cache=cache
        if self.cache == True:
            try:
                os.makedirs('./cache')
            except:
                pass
        self.sprint = sprint()
        self.slog = slog('stock info')
        self.start_date = start_date
        self.end_date = end_date
        self.frequency = frequency
        if type(names) == str:
            self.names = [names]
        elif type(names) == list:
            self.names = names
        self.sprint.hide()
        bs.login()
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)

    @property
    def stock_pair(self):
        self.sprint.pink('getting stock pair...')
        lists = []
        for i in tqdm(self.names):
            rs = bs.query_stock_industry()
            rs = bs.query_stock_basic(code_name=i)
            industry_list = []
            while (rs.error_code == '0') & rs.next():
                industry_list.append(rs.get_row_data())
            info = pd.DataFrame(industry_list, columns=rs.fields)
            lists.append(info)
        df = pd.concat(lists)
        stocks = {}
        for i,j in df[['code','code_name','ipoDate']].iterrows():
            if j[2] >= self.start_date:
                self.sprint.red(f"{j[1]}'s ipo date is {j[2]},which is after {self.start_date}.\nPlease check your start date!")
                raise ValueError("The start date should be after the ipo date!")
            stocks[j[1]] = j[0]
            self.slog.log(f'{j[1]}——{j[0]}',mode=1)
        return stocks  # {'贵州茅台': 'sh.600519', '隆基股份': 'sh.601012'}

    @property
    def stock_data(self, info="date,code,open,close,volume,amount,adjustflag,turn,pctChg"):

        stock_pair = self.stock_pair
        codes = list(stock_pair.values())
        Name = dict(zip(stock_pair.values(), stock_pair.keys())) #交换stock_pair键值对
        lists = []
        self.sprint.yellow('getting stock data...')
        for i in tqdm(codes):
            rs = bs.query_history_k_data_plus(i, info,
                                            start_date=self.start_date, end_date=self.end_date,
                                            frequency=self.frequency, adjustflag="3")
            data_list = []
            while (rs.error_code == '0') & rs.next():
                data_list.append(rs.get_row_data())
            result = pd.DataFrame(data_list, columns=rs.fields)
            result['name'] = Name[i]
            lists.append(result)
        df = pd.concat(lists)
        df=df[['date','name','code','open','close','volume','amount','adjustflag','turn','pctChg']]
        if self.cache == True:
            df.to_csv(f'cache\\{self.names}.csv',encoding='utf8',index=False)
        return (codes, #['sh.600519', 'sh.601012']
                stock_pair, #{'贵州茅台': 'sh.600519', '隆基股份': 'sh.601012'}
                df)

    def quit(self):
        self.sprint.hide()
        bs.logout()

class HolidayStockData(object):
    
    def __init__(self, names=['贵州茅台', '隆基股份'],
                start_date='20200101',
                end_date='20210101',
                frequency='d',
                holiday='国庆节', before=-21, after=21,
                cache=False):
        self.cache = cache
        if self.cache == True:
            try:
                os.makedirs('./cache')
            except:
                pass
        self.sprint = sprint()
        self.slog = slog('holiday stock info')
        self.holiday = holiday
        self.before = before
        self.after = after
        self.names = names
        self.start_date = start_date
        self.end_date = end_date
        self.frequency = frequency
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)
        self.slog.log(f'holiday: {self.holiday}', mode=1)
    @property
    def HolidayNearbyDate(self):
        gd = GetDate(start=self.start_date, end=self.end_date)
        Date = gd.Date
        Date = Date[Date['holiday'] == self.holiday]
        if Date.empty == True:
            self.sprint.red(f'There are no festivals during {self.start_date}--{self.end_date}')
            raise ValueError('There are no festivals during this period!')
        start = []
        year = 0
        for i, j in Date.iterrows():
            if j[6] != year:
                start.append(str(j[0])[0:10])
            year = j[6]
        end = []
        year = list(Date['year'])[0]
        lastdate = None  # !!!!!!!!!!!!!!!!!!!!!!!!
        for i, j in Date.iterrows():
            if j[6] != year:
                end.append(str(lastdate)[0:10])
            year = j[6]
            lastdate = j[0]
        end.append(str(list(Date['date'])[-1])[0:10])

        def func(date, num):
            d = datetime.datetime.strptime(date, '%Y-%m-%d')
            delta = datetime.timedelta(days=num)
            d = d+delta
            return d.strftime('%Y-%m-%d')
        startbefore = []
        for i in start:
            startbefore.append(func(i, self.before))
        endafter = []
        for i in end:
            endafter.append(func(i, self.after))
        df = pd.DataFrame([startbefore, start, end, endafter], index=[
            'start before', 'start', 'end', 'end after'])
        df = df.T
        return df
    
    @property
    def HolidayNearbyData(self):
        date = self.HolidayNearbyDate
        lens = date.index.stop
        result = []
        global StockData
        for i in range(0, lens):
            start_date = list(date['start before'])[i]
            end_date = list(date['end after'])[i]
            StockData = StockData(
                names=self.names, start_date=start_date, end_date=end_date, frequency=self.frequency)
            data = StockData.stock_data
            result.append(data[2])
        df = pd.concat(result)
        StockData.quit()
        self.sprint.show()
        if self.cache == True:
            df.to_csv(f'cache\\{self.names}.csv',encoding='utf8',index=False)
        return (data[0], data[1], df)
    
    def quit(self):
        self.sprint.hide()
        bs.logout()      

if __name__ == '__main__':
    #StockData = StockData(names=['d'],cache=True)
    #print(StockData.stock_pair)
    HolidayStockData=HolidayStockData()
    print(HolidayStockData.HolidayNearbyData)

class StockAbility(object):

    def __init__(self, names=['贵州茅台', '隆基股份'],
                start_year=2018, start_quater=1,
                end_year=2019, end_quater=4):
        self.sprint = sprint()
        self.slog = slog('stock ability')
        self.sprint.red('Please make sure that all the data has already existed!')
        self.names = names
        self.start_year = start_year
        self.end_year = end_year
        self.start_quater = start_quater
        self.end_quater = end_quater
        self.slog.log(f'time: {self.start_year},{self.start_quater} ~~ {self.end_year},{self.end_quater}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        Range = []
        if end_year-start_year >= 2:
            for i in range(start_quater, 5):
                Range.append([start_year, i])
            for i in range(start_year+1, end_year):
                for j in range(1, 5):
                    Range.append([i, j])
            for i in range(1, end_quater+1):
                Range.append([end_year, i])
        elif end_year == start_year:
            for i in range(start_quater, end_quater+1):
                Range.append([end_year, i])
        else:
            for i in range(start_quater, 5):
                Range.append([start_year, i])
            for i in range(1, end_quater+1):
                Range.append([end_year, i])
        self.Range = Range
        self.StockData = StockData(names=self.names)
        self.stock_pair = self.StockData.stock_pair

    # 盈利能力
    @property
    def profit(self):
        intro='''
季频盈利能力
code	证券代码	
pubDate	公司发布财报的日期	
statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
roeAvg	净资产收益率(平均)(%)	归属母公司股东净利润/[(期初归属母公司股东的权益+期末归属母公司股东的权益)/2]*100%
npMargin	销售净利率(%)	净利润/营业收入*100%
gpMargin	销售毛利率(%)	毛利/营业收入*100%=(营业收入-营业成本)/营业收入*100%
netProfit	净利润(元)	
epsTTM	每股收益	归属母公司股东的净利润TTM/最新总股本
MBRevenue	主营营业收入(元)	
totalShare	总股本	
liqaShare	流通股本
        '''	
        self.slog.log(intro)
        profit = []
        self.sprint.blue('getting profit data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                profit_list = []
                profit_list.append(i)
                profit_list.append(k[0])
                profit_list.append(k[1])
                rs_profit = bs.query_profit_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_profit.error_code == '0') & rs_profit.next():
                    profit_list = profit_list+rs_profit.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_profit.fields
                result_profit = pd.DataFrame([profit_list], columns=columns)
                profit.append(result_profit)
        df = pd.concat(profit)
        return df

    # 营运能力
    @property
    def operation(self):
        intro='''
季频营运能力
code	证券代码	
pubDate	公司发布财报的日期	
statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
NRTurnRatio	应收账款周转率(次)	营业收入/[(期初应收票据及应收账款净额+期末应收票据及应收账款净额)/2]
NRTurnDays	应收账款周转天数(天)	季报天数/应收账款周转率(一季报：90天，中报：180天，三季报：270天，年报：360天)
INVTurnRatio	存货周转率(次)	营业成本/[(期初存货净额+期末存货净额)/2]
INVTurnDays	存货周转天数(天)	季报天数/存货周转率(一季报：90天，中报：180天，三季报：270天，年报：360天)
CATurnRatio	流动资产周转率(次)	营业总收入/[(期初流动资产+期末流动资产)/2]
AssetTurnRatio	总资产周转率	营业总收入/[(期初资产总额+期末资产总额)/2]
        '''
        self.slog.log(intro)
        operation = []
        self.sprint.blue('getting operation data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                operation_list = []
                operation_list.append(i)
                operation_list.append(k[0])
                operation_list.append(k[1])
                rs_operation = bs.query_operation_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_operation.error_code == '0') & rs_operation.next():
                    operation_list = operation_list+rs_operation.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_operation.fields
                result_operation = pd.DataFrame(
                    [operation_list], columns=columns)
                operation.append(result_operation)
        df = pd.concat(operation)
        return df

    # 成长能力
    @property
    def growth(self):
        intro='''
季频成长能力
code	证券代码	
pubDate	公司发布财报的日期	
statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
YOYEquity	净资产同比增长率	(本期净资产-上年同期净资产)/上年同期净资产的绝对值*100%
YOYAsset	总资产同比增长率	(本期总资产-上年同期总资产)/上年同期总资产的绝对值*100%
YOYNI	净利润同比增长率	(本期净利润-上年同期净利润)/上年同期净利润的绝对值*100%
YOYEPSBasic	基本每股收益同比增长率	(本期基本每股收益-上年同期基本每股收益)/上年同期基本每股收益的绝对值*100%
YOYPNI	归属母公司股东净利润同比增长率	(本期归属母公司股东净利润-上年同期归属母公司股东净利润)/上年同期归属母公司股东净利润的绝对值*100%
        '''
        self.slog.log(intro)
        growth = []
        self.sprint.blue('getting growth data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                growth_list = []
                growth_list.append(i)
                growth_list.append(k[0])
                growth_list.append(k[1])
                rs_growth = bs.query_growth_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_growth.error_code == '0') & rs_growth.next():
                    growth_list = growth_list+rs_growth.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_growth.fields
                result_growth = pd.DataFrame([growth_list], columns=columns)
                growth.append(result_growth)
        df = pd.concat(growth)
        return df
    
    # 偿债能力
    @property
    def balance(self):
        intro='''
季频偿债能力
code	证券代码	
pubDate	公司发布财报的日期	
statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
currentRatio	流动比率	流动资产/流动负债
quickRatio	速动比率	(流动资产-存货净额)/流动负债
cashRatio	现金比率	(货币资金+交易性金融资产)/流动负债
YOYLiability	总负债同比增长率	(本期总负债-上年同期总负债)/上年同期中负债的绝对值*100%
liabilityToAsset	资产负债率	负债总额/资产总额
assetToEquity	权益乘数	资产总额/股东权益总额=1/(1-资产负债率)        
        '''
        self.slog.log(intro)
        balance = []
        self.sprint.blue('getting balance data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                balance_list = []
                balance_list.append(i)
                balance_list.append(k[0])
                balance_list.append(k[1])
                rs_balance = bs.query_balance_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_balance.error_code == '0') & rs_balance.next():
                    balance_list = balance_list+rs_balance.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_balance.fields
                result_balance = pd.DataFrame([balance_list], columns=columns)
                balance.append(result_balance)
        df = pd.concat(balance)
        return df    

    # 现金流量
    @property
    def cash_flow(self):
        intro='''
季频现金流量
code	证券代码	
pubDate	公司发布财报的日期	
statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
CAToAsset	流动资产除以总资产	
NCAToAsset	非流动资产除以总资产	
tangibleAssetToAsset	有形资产除以总资产	
ebitToInterest	已获利息倍数	息税前利润/利息费用
CFOToOR	经营活动产生的现金流量净额除以营业收入	
CFOToNP	经营性现金净流量除以净利润	
CFOToGr	经营性现金净流量除以营业总收入
        '''
        self.slog.log(intro)
        cash_flow = []
        self.sprint.blue('getting cash flow data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                cash_flow_list = []
                cash_flow_list.append(i)
                cash_flow_list.append(k[0])
                cash_flow_list.append(k[1])
                rs_cash_flow = bs.query_cash_flow_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_cash_flow.error_code == '0') & rs_cash_flow.next():
                    cash_flow_list = cash_flow_list+rs_cash_flow.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_cash_flow.fields
                result_cash_flow = pd.DataFrame(
                    [cash_flow_list], columns=columns)
                cash_flow.append(result_cash_flow)
        df = pd.concat(cash_flow)
        return df
    
    # 杜邦指数
    @property
    def dupont_data(self):
        intro='''
季频杜邦指数
code	证券代码	
pubDate	公司发布财报的日期	
statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
dupontROE	净资产收益率	归属母公司股东净利润/[(期初归属母公司股东的权益+期末归属母公司股东的权益)/2]*100%
dupontAssetStoEquity	权益乘数，反映企业财务杠杆效应强弱和财务风险	平均总资产/平均归属于母公司的股东权益
dupontAssetTurn	总资产周转率，反映企业资产管理效率的指标	营业总收入/[(期初资产总额+期末资产总额)/2]
dupontPnitoni	归属母公司股东的净利润/净利润，反映母公司控股子公司百分比。如果企业追加投资，扩大持股比例，则本指标会增加。	
dupontNitogr	净利润/营业总收入，反映企业销售获利率	
dupontTaxBurden	净利润/利润总额，反映企业税负水平，该比值高则税负较低。净利润/利润总额=1-所得税/利润总额	
dupontIntburden	利润总额/息税前利润，反映企业利息负担，该比值高则税负较低。利润总额/息税前利润=1-利息费用/息税前利润
dupontEbittogr	息税前利润/营业总收入，反映企业经营利润率，是企业经营获得的可供全体投资人（股东和债权人）分配的盈利占企业全部营收收入的百分比
        '''
        self.slog.log(intro)
        dupont_data = []
        self.sprint.blue('getting dupont data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                dupont_data_list = []
                dupont_data_list.append(i)
                dupont_data_list.append(k[0])
                dupont_data_list.append(k[1])
                rs_dupont_data = bs.query_dupont_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_dupont_data.error_code == '0') & rs_dupont_data.next():
                    dupont_data_list = dupont_data_list+rs_dupont_data.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_dupont_data.fields
                result_dupont_data = pd.DataFrame(
                    [dupont_data_list], columns=columns)
                dupont_data.append(result_dupont_data)
        df = pd.concat(dupont_data)
        return df
    
    @property
    def AllAbility(self):
        profit = self.profit
        operation = self.operation
        growth = self.growth
        balance = self.balance
        cash_flow = self.cash_flow
        dupont_data = self.dupont_data
        lists = [growth, balance, cash_flow, dupont_data]
        df = pd.merge(profit, operation, how='outer', on=[
                    'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
        for i in lists:
            df = pd.merge(df, i, how='outer', on=[
                    'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
        return df






import pretty_errors
import pandas as pd
from tqdm import tqdm
from tqdm import trange
import numpy as np
import random
import scipy.optimize as sco
import time
import sys
from .picture import scatter
from .picture import pie
from .GetData import StockData
from .GetData import HolidayStockData
import os
sys.path.append(os.path.dirname(__file__) + os.sep + '../')
from ..log.SlyLog import sprint
from ..log.SlyLog import slog


class calculate(object):
    def __init__(self):
        pass

    def correlation(self, x, y):
        if type(x) == list:
            x = np.array(x)
        if type(y) == list:
            y = np.array(y)
        return (np.dot([xi - np.mean(x) for xi in x], [yi - np.mean(y) for yi in y])/(len(x)-1))/((x.std())*(y.std()))

    def weight(self, lists=['A', 'B', 'C'], mode='dict'):
        if mode == 'dict':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return rand
        elif mode == 'list':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return list(rand.values())


class Markowit(calculate):
    def __init__(self, names=['贵州茅台', '隆基股份'],
                 start_date='2020-12-01',
                 end_date='2020-12-31',
                 frequency="d",
                 holiday=False,
                 holiday_name=None,
                 before=-21, after=21,
                 no_risk_rate=0.45/5200
                 ):
        '''
        马科维兹投资组合
        '''
        self.sprint = sprint()
        self.slog = slog('Markowit')
        self.no_risk_rate = no_risk_rate
        self.frequency = frequency
        self.names = names
        self.start_date = start_date
        self.end_date = end_date
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)
        self.slog.log(
            f'risk-free interest rate: {round(self.no_risk_rate,6)}', mode=1)
        try:
            os.makedirs('./picture')
        except:
            pass
        cache = False
        while cache == False:
            try:
                self.stock_data = pd.read_csv(f'cache\\{self.names}.csv')
                self.sprint.cyan('Use cache data to calculate!')
                break
            except:
                self.sprint.cyan('No cache data found!')
                cache = True
                self.sprint.red(
                    'Please make sure that all of the stocks are in the market!')
                self.sprint.green('initializing...')
                if holiday == False:
                    self.StockData = StockData(names=names,
                                               start_date=start_date,
                                               end_date=end_date,
                                               frequency=frequency,
                                               cache=cache)
                    self.codes, self.stock_pair, self.stock_data = self.StockData.stock_data
                    self.StockData.quit()
                else:
                    if holiday_name == None:
                        holiday_name = '国庆节'
                    self.holiday_name = holiday_name
                    start_date = self.start_date.replace('-', '')
                    end_date = self.end_date.replace('-', '')
                    self.HolidayStockData = HolidayStockData(names=names,
                                                             start_date=start_date,
                                                             end_date=end_date,
                                                             holiday=holiday_name,
                                                             frequency=frequency,
                                                             before=before, after=after,
                                                             cache=cache)
                    self.codes, self.stock_pair, self.stock_data = self.HolidayStockData.HolidayNearbyData
                break

    def sharp(self, weights=[], stock_list=[]):
        if weights == []:
            weights = len(self.names) * [1. / len(self.names), ]
        if stock_list == []:
            stock_list = self.names
        df_temp = self.stock_data[['pctChg', 'name']]
        df = df_temp[['name']]
        df['pctChg'] = df_temp['pctChg'].astype('float')
        del df_temp
        covs = []
        for i in range(0, len(stock_list)):
            for j in stock_list[i:]:
                if stock_list[i] != j:
                    covs.append([stock_list[i], j])
        for i in covs:
            x = list(df[df['name'] == i[0]]['pctChg'])
            y = list(df[df['name'] == i[1]]['pctChg'])
            i.append(self.correlation(x, y))
        dic = {}
        rate = 0
        risk = 0
        c = 0
        rand = {}
        for i in stock_list:
            weight = weights[c]
            rand[i] = weight
            data = list(df[df['name'] == i]['pctChg'])
            Mean = np.mean(data)
            Std = np.std(data)
            rate += Mean*weight
            risk += (weight*Std)**2
            dic[i] = weight*Std
            c += 1
        for i in covs:
            risk += 2*i[2]*dic[i[0]]*dic[i[1]]
        sharp = (rate-self.no_risk_rate)/risk
        return {'sharp': sharp, 'rate': rate, 'risk': risk, 'weight': rand}

    def max_sharp(self, weights, *args):
        stock_list = args[0]
        self.slog.log(f'weights: {weights}', mode=1)
        return -self.sharp(weights=weights, stock_list=stock_list)['sharp']

    def portfolio(self, stock_list=[], accurate=True, number=500):
        if stock_list == []:
            stock_list = self.names
        self.sprint.magenta(f'building a portfolio for {stock_list}')
        try:
            os.makedirs('./result')
        except:
            pass
        if accurate == True:
            opts = sco.minimize(fun=self.max_sharp,
                                x0=len(stock_list) * [1. / len(stock_list), ],
                                method='SLSQP',
                                args=(stock_list,),
                                bounds=tuple((0, 1)
                                             for x in range(len(stock_list))),
                                constraints={'type': 'eq',
                                             'fun': lambda x: np.sum(x) - 1}
                                )
            count = 0
            result = {}
            ans = list(opts['x'])
            for i in stock_list:
                result[i] = ans[count]
                count += 1
            pie(dic=result, path='picture\\weights-pie')
            result = [result, -opts['fun']]
            self.slog.log(f'weights:{result[0]}\nsharp:{result[1]}')
            df = pd.DataFrame([result], columns=['weights', 'sharp'])
            df.to_csv('result\\accurate_result.csv',
                      index=False, encoding='utf8')
            return result
        else:
            lists = []
            for i in trange(number):
                weights = self.weight(lists=stock_list, mode='list')
                sharp = self.sharp(
                    weights=weights, stock_list=stock_list)
                lists.append(list(sharp.values()))
            df = pd.DataFrame(
                lists, columns=['sharp', 'rate', 'risk', 'weight'])
            df = df.sort_values('sharp', ascending=False)
            data = []
            for i, j in df.iterrows():
                data.append([j[1], j[2]])
            scatter(data=data, name=[
                    str(self.names)], title='sharp-scatter', path='picture\\sharp-scatter')
            weights = list(df['weight'])[0]
            sharp = list(df['sharp'])[0]
            pie(dic=weights, path='picture\\weights-pie')
            self.slog.log(f'weights:{weights}\nsharp:{sharp}')
            df.to_csv('result\\not_accurate_result.csv',
                      index=False, encoding='utf8')
            return df


if __name__ == '__main__':
    Markowit = Markowit(names=['隆基股份', '五粮液', '贵州茅台'], start_date='2019-12-01',
                        end_date='2020-12-31',
                        frequency='w',
                        holiday=False,
                        )
    print(Markowit.portfolio(accurate=True))



__name__ = "Suluoya"
__author__ = 'Suluoya'
__all__ = ['Markowit','GetGoodStock','GetData','GetDate','picture']
from .Markowit import *
from .GetGoodStock import *
from .GetData import *
from .GetDate import *
from .picture import *


import pygal
from pyecharts import options as opts
from pyecharts.charts import EffectScatter
from pyecharts.globals import SymbolType
import pyecharts.options as opts
from pyecharts.charts import Pie
from pyecharts.charts import Scatter
import numpy as np
import pandas as pd
import matplotlib
#matplotlib.rcParams['backend'] = 'HTML'
import matplotlib.pyplot as plt
def weights_pie(weights):
    #weights={'伊利股份': 0.665225115503282, '新华保险': 0.20896815868555363, '中国建筑': 0.12580672581116445}
    lab = list(weights.keys()) 
    num = list(weights.values())
    (
        Pie(init_opts=opts.InitOpts(width='1440px', height='729px')) #自定义画布大小
        .add(series_name='', data_pair=[(i, j)for i, j in zip(lab, num)],radius=['40%', '75%']) #遍历数据
        .set_global_opts(title_opts=opts.TitleOpts(title="weights",subtitle="")) #添加主、副标题
        .set_series_opts(label_opts=opts.LabelOpts(formatter="{b}: {d}%")) #添加数据标签   

    ).render('Markowit\\weights.html')

def effect_scatter(datax,datay):
    v1 =datax
    v2 =datay
    c = (
        EffectScatter()
        .add_xaxis(v1)
        .add_yaxis('',v2)
            .set_global_opts(title_opts=opts.TitleOpts(title="EffectScatter"))
        )
    c.render('scatter.html')


#饼图
def pie(dic=None,path='pie'):
    pie=pygal.Pie(inner_radius=0.6)
    if dic==None:
        pie.add("优秀",19)
        pie.add("良好",36)
        pie.add("合格",36)
        pie.add("不合格",4)
        pie.add("较差",2)
    elif type(dic)==dict:
        for i,j in dic.items():
            pie.add(i,j)
    
    pie.render_to_file(f'{path}.svg')

# stroke参数是指是否禁用连线
def scatter(data=[[1,3],[2,4]],name=['Suluoya'],title='Suluoya',path='scatter-plot'):
    xy_chart = pygal.XY(stroke=False)
    xy_chart.title = title
    for i in name:
        xy_chart.add(i, data)
    xy_chart.render_to_file(path+'.svg')
if __name__ == '__main__':
    scatter()
#pie(dic={'伊利股份': 0.665225115503282, '新华保险': 0.20896815868555363, '中国建筑': 0.12580672581116445},path='try')

#
##Bar()绘制条形图
#hist = pygal.Bar()
#x= ['1', '2', '3', '4', '5', '6']
#y=[10,30,15,35,67,43]
#hist.x_labels =x
#hist.x_title = "Result"
#hist.y_title = "Number"
#hist.add('条形图', y)
#hist.render_to_file('bar.svg')

#hist=pygal.Bar(margin_bottom=10,#图与低端距离，类似的有上下左右
#                  height=450,
#                  #style=NeonStyle,#设置绘图风格，pygal拥有23种style，
#                  #其它style可选：'BlueStyle', 'CleanStyle', 'DarkColorizedStyle', 'DarkGreenBlueStyle', 'DarkGreenStyle', 'DarkSolarizedStyle', 'DarkStyle', 'DarkenStyle', 'DefaultStyle', 'DesaturateStyle', 'LightColorizedStyle', 'LightGreenStyle', 'LightSolarizedStyle', 'LightStyle', 'LightenStyle', 'NeonStyle', 'ParametricStyleBase', 'RedBlueStyle', 'RotateStyle', 'SaturateStyle', 'SolidColorStyle', 'Style', 'TurquoiseStyle'
#                  
#                  ##title设置
#                  title=u'Some points', #图标题
#                  x_title='X Axis',#x轴标题
#                  y_title='Y Axis',#y轴标题
#                  
#                  ##label设置
#                  show_x_labels=True,#显示x轴标签
#                  x_label_rotation=20,#x轴标签倾斜角度
#                  x_labels = list('ABCD'),#自定义x轴标签
#                  value_formatter = lambda x: "%.2f" % x,#y轴刻度值格式化输出
#                  
#                  ##图例legend设置
#                  show_legend=True,#开启图例
#                  legend_at_bottom=True,#图例放置于底部
#                  legend_at_bottom_columns=2,#图例标签显示行数
#                  legend_box_size=12,#图例前箱子大小
#                  
#                  ##坐标轴axis设置
#                  include_x_axis=True,#坐标轴开启
#                  range=(0, 30),#设置y轴刻度值范围
#                  
#                  secondary_range=(10, 25),#第二坐标轴刻度范围
#                  xrange=(0,10),#x轴刻度范围
#                  
#                  ##柱子上text设置
#                  print_values=True,#开启柱子上文本
#                  print_values_position='top',#文本位置
#                  style=LightSolarizedStyle(
#                  value_font_family='googlefont:Raleway',#文本字体设置
#                  value_font_size=15,#大小
#                  value_colors=('red','blue'),#颜色设置
#                  ),
#                  
#                 )
#
#
##HorizontalBar()创建水平条形图
#bar=pygal.HorizontalBar()
#bar.title="小学生成绩分布图"
#bar.add("优秀",19)
#bar.add("良好",36)
#bar.add("合格",36)
#bar.add("不合格",4)
#bar.add("较差",2)
#bar.render_to_file('bar1.svg')
#
#
##HorizontalStackedBar()绘制水平堆叠条形图
#bar2=pygal.HorizontalStackedBar()
#bar2.title='员工销售情况图'
#x=[2,3,6,5,9,16,18]
#y=[1,2,3,5,6,2,7]
#bar2.x_labels=["小二","张三","李四","小桐","王五","小小","赵六"]
#bar2.x_title='销售量'
#bar2.y_title='人员'
#bar2.add('旺季',x)
#bar2.add('淡季',y)
#bar2.render_to_file('bar2.svg')
#
##XY()绘制函数图像
##数学公式这些要用到math模块
#from math import cos 
#xy_chart = pygal.XY()
#xy_chart.title = '函数图'
##需提供一个横纵坐标元组作为元素的列表,即提供几个（x,y）坐标点的列表
#xy_chart.add('x = cos(y)', [(cos(x / 10.), x / 10.) for x in range(-50, 50, 5)])
#xy_chart.add('y = cos(x)', [(x / 10., cos(x / 10.)) for x in range(-50, 50, 5)])
#xy_chart.add('x = 1', [(1, -5), (1, 5)])
#xy_chart.add('x = -1', [(-1, -5), (-1, 5)])
#xy_chart.add('y = 1', [(-5, 1), (5, 1)])
#xy_chart.add('y = -1', [(-5, -1), (5, -1)])
#xy_chart.render_to_file('bar_chart.svg')
#
#
#Line()绘制折线图

#line=pygal.Line()
#x=[2,3,6,5,9,16,18]
#y=[1,2,3,5,6,2,7]
#line.x_labels=map(str,range(2008,2014))
#line.add('旺季',x)
#line.add('淡季',y)
#line.render_to_file('line.svg')
#
##HorizontalLine()绘制水平折线图
#line1=pygal.HorizontalLine()
#line1.title='销售线图'
#line1.x_labels=map(str,range(2008,2014))
#x=[2,3,6,5,9,16,18]
#y=[1,2,3,5,6,2,7]
#line1.add('旺季',x)
#line1.add('淡季',y)
#line1.render_to_file('line1.svg')
#
##StackedLine(fill=True)绘制叠加侧线图
#line2=pygal.StackedLine(fill=True)
#line2.title='叠加侧线图'
#line2.x_labels=map(str,range(2008,2014))
#x=[2,3,6,5,9,16,18]
#y=[1,2,3,5,6,2,7]
#line2.add('旺季',x)
#line2.add('淡季',y)
#line2.render_to_file('line2.svg')
#
##Radar()绘制雷达图
#radar=pygal.Radar()
#x=[2,3,6,5,9,16,18]
#y=[1,2,3,5,6,2,7]
#radar.title='淡旺季雷达分布图'
#radar.add('旺季',x)
#radar.add('淡季',y)
#radar.render_to_file('radar.svg')
#
##Histogram()绘制直方图
##使用函数Histogram()绘制直方图是一个特殊的条形图，它包含3个数值：纵坐标高度，横坐标开始和横坐标结束。
#gram=pygal.Histogram()
#gram.title='直方图'
#gram.add('宽直条',[(4,0,10),(4,5,12),(2,0,16)])
#gram.add('窄直条',[(9,1,2),(12,4,4.6)])
#gram.render_to_file('gram.svg')